//  Copyright (c) 2005-2009 Hartmut Kaiser
//
//  Distributed under the Boost Software License, Version 1.0. (See accompanying
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if !defined(BOOST_PP_IS_ITERATING)

#if !defined(SAGA_IMPL_ENGINE_RUN_MODE_WRAPPER_IMPL_HPP)
#define SAGA_IMPL_ENGINE_RUN_MODE_WRAPPER_IMPL_HPP

#include <saga/impl/config.hpp>

#include <boost/preprocessor/iterate.hpp>
#include <boost/preprocessor/repetition/enum_params.hpp>
#include <boost/preprocessor/repetition/enum_binary_params.hpp>

#define BOOST_PP_ITERATION_PARAMS_1                                           \
    (3, (1, SAGA_ARGUMENT_LIMIT,                                              \
    "saga/impl/engine/run_mode_wrapper_impl.hpp"))                                 \
    /**/

#include BOOST_PP_ITERATE()

#endif // !SAGA_IMPL_ENGINE_RUN_MODE_WRAPPER_IMPL_HPP

///////////////////////////////////////////////////////////////////////////////
//
//  Preprocessor vertical repetition code
//
///////////////////////////////////////////////////////////////////////////////
#else // defined(BOOST_PP_IS_ITERATING)

#define K BOOST_PP_ITERATION()

    ///////////////////////////////////////////////////////////////////////////
    template <
        typename Base, typename RetVal,
        BOOST_PP_ENUM_PARAMS(K, typename FuncArg),
        BOOST_PP_ENUM_PARAMS(K, typename Arg)
    >
    inline saga::task
    execute_sync(proxy *prxy,
        char const *cpi_name, char const *name, char const *op_name,
        v1_0::preference_type const& prefs,
        void (Base::*sync)(RetVal&, BOOST_PP_ENUM_PARAMS(K, FuncArg)),
        saga::task (Base::*async)(BOOST_PP_ENUM_PARAMS(K, FuncArg)),
        BOOST_PP_ENUM_BINARY_PARAMS(K, Arg, const& arg))
    {
        adaptor_selector::adaptor_info_list_type no_no_list;
        exception_list exceptions;
        bool found_one = false;
        v1_0::op_info oi (name);
        while (true) {
            v1_0::cpi_info info;
            try {
                run_mode current_mode = Unknown;
                cpi_list::cpi_type current_cpi;

                {
                // try to create a cpi_instance implementing the required
                // function
                    proxy::mutex_type::scoped_lock lock (prxy->mtx_);
                    current_mode = prxy->select_run_mode(cpi_name, name, prefs,
                        true, no_no_list, oi);
                    BOOST_ASSERT(!prxy->cpis_.empty());

                // retrieve info to be able to store it in no_no_list later
                    current_cpi = prxy->cpis_.get_current();
                    info = current_cpi->get_adaptor_info();
                }

                found_one = true;
                return dispatch_sync(current_mode, op_name, current_cpi,
                    sync, async, BOOST_PP_ENUM_PARAMS(K, arg));
            }
            catch (saga::exception const& e) {
                // save this exception, avoid NoAdaptor errors if we actually
                // found at least one
                if (!found_one || e.get_error() != (saga::error)saga::adaptors::NoAdaptor)
                    exceptions.add(e);

                // throw, if we don't find anything
                if (e.get_error() == (saga::error)saga::adaptors::NoAdaptor ||
                    e.get_error() == (saga::error)saga::adaptors::NoAdaptorInfo)
                {
                    saga::error err = exceptions.get_error();
                    if (err == (saga::error)saga::adaptors::NoAdaptorInfo ||
                        err == (saga::error)saga::adaptors::NoAdaptor)
                    {
                        err = saga::NoSuccess;
                    }

                    // The synchronous functions do not throw, any error get's
                    // reported during get_result and/or rethrow
                    saga::task t(saga::task::Failed);
                    return detail::set_task_exception(t, prxy, exceptions);
                }

                // just try the next adaptor
                no_no_list.push_back(info);
            }
            catch (std::exception const& e) {
                // save this exception
                TR1::shared_ptr<saga::impl::proxy> impl(
                    boost::const_pointer_cast<saga::impl::proxy>(
                        prxy->shared_from_this()));

                exceptions.add(saga::exception(runtime::get_object(impl),
                    std::string("std::exception caught") + e.what(), 
                    (saga::error)saga::adaptors::Unexpected));

                // just try the next adaptor
                no_no_list.push_back(info);
            }
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    template <
        typename Base, typename RetVal,
        BOOST_PP_ENUM_PARAMS(K, typename FuncArg),
        BOOST_PP_ENUM_PARAMS(K, typename Arg)
    >
    inline saga::task
    execute_async(proxy *prxy,
        char const *cpi_name, char const *name, char const *op_name,
        v1_0::preference_type const& prefs,
        void (Base::*sync)(RetVal&, BOOST_PP_ENUM_PARAMS(K, FuncArg)),
        saga::task (Base::*async)(BOOST_PP_ENUM_PARAMS(K, FuncArg)),
        BOOST_PP_ENUM_BINARY_PARAMS(K, Arg, const& arg))
    {
        TR1::shared_ptr<adaptor_selector_state> state(
            new adaptor_selector_state(prxy, cpi_name, name, op_name, prefs));

        return dispatch_async(prxy, state, sync, async, BOOST_PP_ENUM_PARAMS(K, arg));
    }

    ///////////////////////////////////////////////////////////////////////////
    template <
        typename Base, typename RetVal,
        BOOST_PP_ENUM_PARAMS(K, typename FuncArg),
        BOOST_PP_ENUM_PARAMS(K, typename Arg)
    >
    inline saga::task
    execute_sync_async(proxy *prxy,
        char const *cpi_name, char const *name, char const *op_name,
        v1_0::preference_type const& prefs, bool is_sync,
        void (Base::*sync)(RetVal&, BOOST_PP_ENUM_PARAMS(K, FuncArg)),
        saga::task (Base::*async)(BOOST_PP_ENUM_PARAMS(K, FuncArg)),
        BOOST_PP_ENUM_BINARY_PARAMS(K, Arg, const& arg))
    {
        return is_sync ?
            execute_sync(prxy, cpi_name, name, op_name, prefs, sync, async,
                BOOST_PP_ENUM_PARAMS(K, arg)) :
            execute_async(prxy, cpi_name, name, op_name, prefs, sync, async,
                BOOST_PP_ENUM_PARAMS(K, arg));
    }

#undef K

#endif // defined(BOOST_PP_IS_ITERATING)
