//  Copyright (c) 2005-2009 Hartmut Kaiser
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <utility>
#include <vector>
#include <string>

#include <boost/assert.hpp>

#include <saga/saga/task.hpp>
#include <saga/saga/url.hpp>

#include <saga/impl/task.hpp>    // include the task implementation we want to use

namespace saga
{
    // implement the get_result functions for the saga::task object
    template <typename Retval>
    Retval& task::get_result()
    {
        if (saga::task_base::Failed == get_task_if()->get_state())
        {
            get_task_if()->rethrow();     // must throw
        }

        Retval* retval = saga::detail::any_cast<Retval>(
            &get_task_result(*this));
        if (0 == retval)
        {
            SAGA_THROW("Wrong data type requested while calling get_result", 
                saga::NoSuccess);
        }
        return *retval;
    }

    template <typename Retval>
    Retval const& task::get_result() const
    {
        if (saga::task_base::Failed == get_task_if()->get_state())
        {
            get_task_if()->rethrow();     // must throw
        }

        Retval const* retval = saga::detail::any_cast<Retval const>(
            &get_task_result(*this));
        if (0 == retval)
        {
            SAGA_THROW("Wrong data type requested while calling get_result", 
                saga::NoSuccess);
        }
        return *retval;
    }
}

